﻿
--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectGroups_GetList' ))
	Drop procedure dbo.A_Policy_SubjectGroups_GetList
GO

Create procedure dbo.A_Policy_SubjectGroups_GetList
	@objecttype int = 0
as 
begin

	-- Возвращает список всех именованных субъектов, доступных для назначения прав

	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#subjects'))
		Drop table #subjects
		
	Create table #subjects ( [type] int, [name] nvarchar(500), [id] uniqueidentifier, [extraId] uniqueidentifier, [equalMode] int )


	-- Пользователи
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 1, [Name], [Id], null, 2
		From A_Users u
		Where u.IsDeleted = 0
	
	-- Отделы
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 2, [Name], [Id], null, 2
		From A_Groups g	
	
	-- Роли
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 3, r.[Name], r.[Id], null, 2
		From A_UserRoles_Roles r
		Where not exists ( Select Id from A_Users u Where u.Id = r.Id )
			and Domination = 0
	
	
	-- Все операторы	
	Declare @alloperators nvarchar(200)
	Select @alloperators = langvalue 
	From A_LangTexts
	Where langkey like 'alloperators'
	if ( @alloperators is NULL )
		Set @alloperators = N'Все операторы'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 4, @alloperators, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 4
		
		
	if ( @objecttype in ( 0, 101, 102, 105, 106, 107 )) begin
	
		-- Операторы текущего проекта
		Declare @currentprojectoperators nvarchar(200)
		Select @currentprojectoperators = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectoperators'
		if ( @currentprojectoperators is NULL )
			Set @currentprojectoperators = N'Операторы текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 12, @currentprojectoperators, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 12
	end
	
	
	---- Операторы задачи
	--Declare @taskoperators nvarchar(200)
	--Select @taskoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'taskoperators'
	--if ( @taskoperators is NULL )
	--	Set @taskoperators = 'Операторы задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @taskoperators + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Операторы проекта
	--Declare @projectoperators nvarchar(200)
	--Select @projectoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectoperators'
	--if ( @projectoperators is NULL )
	--	Set @projectoperators = 'Операторы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @projectoperators + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	
	
	-- Все супервизоры
	Declare @allsupervisors nvarchar(200)
	Select @allsupervisors = langvalue 
	From A_LangTexts
	Where langkey like 'allsupervisors'
	if ( @allsupervisors is NULL )
		Set @allsupervisors = N'Все супервизоры'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 7, @allsupervisors, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 7
	
	
	if ( @objecttype in ( 0, 101, 102, 105, 106, 107 )) begin
	
		-- Супервизоры текущего проекта	
		Declare @currentprojectsupervisors nvarchar(200)
		Select @currentprojectsupervisors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectsupervisors'
		if ( @currentprojectsupervisors is NULL )
			Set @currentprojectsupervisors = N'Супервизоры текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 13, @currentprojectsupervisors, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 13	
	end
	
	---- Супервизоры задачи
	--Declare @tasksupervisors nvarchar(200)
	--Select @tasksupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'tasksupervisors'
	--if ( @tasksupervisors is NULL )
	--	Set @tasksupervisors = 'Супервизоры задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 8, @tasksupervisors + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Супервизоры проекта
	--Declare @projectsupervisors nvarchar(200)
	--Select @projectsupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectsupervisors'
	--if ( @projectsupervisors is NULL )
	--	Set @projectsupervisors = 'Супервизоры проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 9, @projectsupervisors + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	

	---- Владельцы проекта
	--Declare @projectowners nvarchar(200)
	--Select @projectowners = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectowners'
	--if ( @projectowners is NULL )
	--	Set @projectowners = 'Владельцы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 10, @projectowners + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p


	-- По другим привилегиями других объектов - ниже
	
	
	-- Все пользователи
	Declare @allusers nvarchar(200)
	Select @allusers = langvalue 
	From A_LangTexts
	Where langkey like 'allusers'
	if ( @allusers is NULL )
		Set @allusers = N'Все пользователи'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 14, @allusers, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 14


	-- По привилегиям всех объектов - слишком много попадет в список. Пропустим пока.
	-- Только на проекты сошлемся
	
	if ( @objecttype in ( 0, 102, 105, 106, 107 )) begin
		
		Declare @idempty uniqueidentifier
		Set @idempty = '00000000-0000-0000-0000-000000000000'
		
		
		-- Читатели текущего проекта
		Declare @currentprojectviewers nvarchar(200)
		Select @currentprojectviewers = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectviewers'
		if ( @currentprojectviewers is NULL )
			Set @currentprojectviewers = N'Пользователи с правом просмотра текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectviewers, @idempty, 'F876E650-7972-475A-AE96-BCCF831AF818', 4
					
						
		-- Редакторы текущего проекта
		Declare @currentprojectredactors nvarchar(200)
		Select @currentprojectredactors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectredactors'
		if ( @currentprojectredactors is NULL )
			Set @currentprojectredactors = N'Пользователи с правом изменения текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectredactors, @idempty, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', 4
						
	end
	
	
	-- Итоговый вывод
	Select s.[type], tp.[priority], s.[name], s.[id], s.[extraId], s.[equalMode]
	From #subjects s
		Inner join dbo.A_Policy_SubjectTypePriority tp on tp.[idType] = s.[type]
	Order By tp.[priority], s.[name]
	
end
GO

--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectPrivileges_GetByObject' ))
	Drop procedure dbo.A_Policy_SubjectPrivileges_GetByObject
GO

Create procedure dbo.A_Policy_SubjectPrivileges_GetByObject
	@idobject uniqueidentifier,
	@actionUserId uniqueidentifier = NULL	
as 
begin
	-- Возвращает список установленных привилегий (всех - и разрешенных, и запрещенных) для указанного объекта
	--  без учета вложенности привилегий по разным объектам
	
	-- Общие группы, названия из языковой таблицы
	Declare @groupnames table ( [id] int, [extraId] uniqueidentifier, [code] nvarchar(200) collate database_default, [name] nvarchar(500) collate database_default)

	Insert into @groupnames ( [id], [code], [name] ) Values ( 4, 'alloperators', N'Все операторы' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 7, 'allsupervisors', N'Все супервизоры' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 12, 'currentprojectoperators', N'Операторы текущего проекта' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 13, 'currentprojectsupervisors', N'Супервизоры текущего проекта' )
	Insert into @groupnames ( [id], [code], [name] ) Values ( 14, 'allusers', N'Все пользователи' )
	Insert into @groupnames ( [id], [extraId], [code], [name] ) Values ( 101, 'F876E650-7972-475A-AE96-BCCF831AF818', N'currentprojectviewers', 'Пользователи с правом просмотра текущего проекта' )
	Insert into @groupnames ( [id], [extraId], [code], [name] ) Values ( 101, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', N'currentprojectredactors', 'Пользователи с правом изменения текущего проекта' )

	Update g
	Set g.[name] = l.[langvalue]
	From @groupnames g, A_LangTexts l
	Where g.[code] = l.[langkey]
		
			
	-- Права пользователя на изменение привилегий
	Declare @resmodifyall int
	Declare @userprivileges table ( [idPrivilege] uniqueidentifier, [isHigherPrivilege] bit )
	
	Set @resmodifyall = 0
	if ( not @actionUserId is null ) begin
	
		/* Привилегия изменения прав для этого объекта есть?
		*/
		exec A_Policy_SubjectPrivileges_CheckUserHasModifyPrivilege 
			@idUser = @actionUserId, 
			@idObject = @idobject,
			@onlyreturnvalue = 1,
			@resultvalue = @resmodifyall OUTPUT
			
		-- Если нет, то выгребаем те привелегии, которые юзер сам имеет
		if ( @resmodifyall = 1 ) begin
		
			-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
			if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
				Drop table #A
				
			Create table #A ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
				
			-- С учетом запретов и приоритетов разных субъектных групповых объединений
			-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
			Insert into #A ( [idObject], [idPrivilege], [subjType], [value] )
				Select sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
				From A_Policy_SubjectPrivileges sp
					Inner join dbo.A_Policy_SubjectGroupCache ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and ((ur.[subjExtraId] is null) or (ur.[subjExtraId] = sp.[subjExtraid]))
				Where (sp.[idObject] = @idObject)
					and (ur.[idUser] = @actionUserId)				
				Group by sp.[idObject], sp.[idPrivilege], ur.[subjType]
						
			-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
			--  На выход нас интересуют разрешенные - те у кого value получился 1
			Insert into @userprivileges ( [idPrivilege], [isHigherPrivilege] )
				Select a.[idPrivilege], 0
				From ( Select a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
						From #A a
							inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
						Group by a.[idObject], a.[idPrivilege]
					)x
					Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
					Inner join #A a on (b.[idType] = a.[subjType]) 
										and (x.[idObject] = a.[idObject]) 
										and (x.[idPrivilege] = a.[idPrivilege])
				Where a.[value] = 1
		end
		
	end
	
	

	-- Выборка итоговая
	Select 
		sp.[subjType],
	
		case when (( sp.[subjType] = 101 ) 
					and ( sp.[subjExtraId] in ('F876E650-7972-475A-AE96-BCCF831AF818', 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3')))
				then NULL
			else sp.[idSubject]
		end as idSubject,
		
		case when ( sp.subjType in ( 12, 13 ))
				then NULL
			else sp.[subjExtraId]
		end as subjPrivilegeId,
		
		sp.[idPrivilege], 
		sp.[value], 
		sp.[isDefault], 
		sp.[idCreator],
		
		cast ( case when ( @resmodifyall = 0 )
					then 0
					when ( @resmodifyall = 1 )
					then IsNull ( up.[isHigherPrivilege], 1 )
					else 1
				end as bit ) as isHigherPrivilege,				
		
		st.[name] as subjTypeName, 
		p.[name] as subjPrivilegeName,		
		
		case when (sp.[subjType] = 1)
				then IsNull ( xusers.Name, '' )
			 when (sp.[subjType] = 2)
				then IsNull ( xdepartments.[Name], '' )
			 when (sp.[subjType] = 3)			 
				then IsNull ( xroles.[Name], '' )
			 when (sp.[subjType] in (5, 8))
				then IsNull ( xtasks.[Name], '' )
			 when (sp.[subjType] in (6, 9, 10))
				then IsNull ( xprojs.[Name], '' )
			 when (sp.[subjType] in (4, 7, 12, 13, 14))
				then IsNull ( xgroupnames.[Name], '' )
			 when ((sp.[subjType] = 101) and (sp.[subjExtraId] = 'F876E650-7972-475A-AE96-BCCF831AF818'))
				then IsNull ( xgroupnames.[Name], '' )
			 when ((sp.[subjType] = 101) and (sp.[subjExtraId] = 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3'))
				then IsNull ( xgroupnames.[Name], '' )
			 when (sp.[subjType] = 101)
				then IsNull ( xobjs.[name], '' )--Здесь из кэша субъектных групп надо взять
			 else NULL
		end as subjGroupName,
		
		case when (sp.[subjType] = 101)
				then IsNull ( xobjtypes.[id], 0 )
			 else NULL
		end as subjObjectType,
		 
		case when (sp.[subjType] = 101)
				then IsNull ( xobjtypes.[name], '' )
			 else NULL
		end as subjObjectTypeName,
		
		case when (sp.[subjType] = 101)
				then IsNull ( xobjs.[name], '' )
			 else NULL
		end as subjObjectName
		 
	From dbo.A_Policy_SubjectPrivileges sp
		Inner join dbo.A_Policy_SubjectTypes as st on st.[id] = sp.[subjType]
		Left join dbo.A_Policy_Privileges as p on p.[id] = sp.[subjExtraId]
		Left join dbo.A_Users as xusers on (sp.[subjType] = 1) and (xusers.[Id] = sp.[idSubject])
		Left join dbo.A_Groups as xdepartments on (sp.[subjType] = 2) and (xdepartments.[Id] = sp.[idSubject])
		Left join dbo.A_UserRoles_Roles as xroles on (sp.[subjType] = 3) and (xroles.[Id] = sp.[idSubject])
		Left join @groupnames as xgroupnames on ((sp.[subjType] in (4, 7, 12, 13, 14)) and (xgroupnames.[id] = sp.[subjType]))
											 or ((sp.[subjType] = 101) and (xgroupnames.[id] = sp.[subjType]) and (xgroupnames.[extraId] = sp.[subjExtraId]))
		Left join dbo.A_TaskManager_Tasks as xtasks on (sp.[subjType] in (5, 8)) and (xtasks.[Id] = sp.[idSubject])
		Left join dbo.A_TaskManager_Projects as xprojs on (sp.[subjType] in (6, 9, 10)) and (xprojs.[Id] = sp.[idSubject])
		--Left join dbo.A_TaskManager_Projects as xprojsx on (sp.[subjType] in (12, 13)) and (xprojs.[Id] = sp.[subjExtraId])
		Left join dbo.A_Policy_Objects xobjs on (sp.[subjType] = 101) and (xobjs.[id] = sp.[idSubject])
		Left join dbo.A_Policy_ObjectTypes xobjtypes on (sp.[subjType] = 101) and (xobjtypes.[id] = xobjs.[type])
		--Left join со всеми таблицами объектов, поддерживающих POLICY, через xobjs.type и xobjs.id
		Left join @userprivileges up on up.[idPrivilege] = sp.[idPrivilege]
	Where (sp.[idObject] = @idobject)

end
GO
