﻿
if exists ( Select * FROM sysobjects where id = OBJECT_ID ( 'dbo.A_Policy_SubjectGroups_GetByAllUsers' ))
	Drop procedure dbo.[A_Policy_SubjectGroups_GetByAllUsers]
GO

Create procedure [dbo].[A_Policy_SubjectGroups_GetByAllUsers]
	@myuser uniqueidentifier = NULL,
	@buildnesting bit = 1,
	@nestinglevels int = 100,
	@fillcache bit = 0
as
begin
		
	-- Возвращает список идентификаторов и типов ролей, в которых участвует пользователь,
	--  чтобы затем пересекать итог с таблицами на определение наличия привилегий.
		
	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#userroles'))
		Drop table #userroles
		
	Create table #userroles 
	( 
		[idUser] uniqueidentifier, 
		[idSubject] uniqueidentifier, 
		[subjType] int, 
		[subjExtraId] uniqueidentifier, 
		[levelNesting] int 
	)

	-- Сам пользователь
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select [ID], [ID], 1
		From dbo.A_Users u		
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[ID]))
		
	-- Отдел
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[ID], g.[ID], 2
		From A_Groups g
			inner join A_Users u on u.[parentgroupid] = g.[ID]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[ID]))

	-- Роли
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], r.[Id], 3
		From A_UserRoles_Roles r
			Inner join A_UserRoles_RolesOfUsers ur on ur.[IdRole] = r.[Id]
			Inner join A_Users u on u.[Id] = ur.[IdUser]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
		
	-- Все операторы (должны иметь также доступ к проектам соответствующим)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[ID], st.[groupId], 4
		From A_UserParams up
			Inner join A_Users u on u.[ID] = up.[IdUser]
			Inner join A_Policy_SubjectTypes st on st.[id] = 4
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[ID]))
			and (up.[IsOperator] = 1)
			
	-- Операторы задачи (должны иметь также доступ к проекту соответствующему)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], t.[Id], 5
		From A_TaskManager_Tasks t
			Inner join A_TaskManager_Operators o on o.[TaskId] = t.[Id]
			Inner join A_UserParams up on up.[IdUser] = o.[OperatorId]
			Inner join A_Users u on u.[Id] = o.[OperatorId]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
			and (up.[IsOperator] = 1)
			
	-- Операторы проекта (должны иметь также доступ к проекту соответствующему)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], p.[Id], 6
		From A_TaskManager_Projects p
			Inner join A_TaskManager_Tasks t on t.[IdProject] = p.[Id]
			Inner join A_TaskManager_Operators o on o.[TaskId] = t.[Id]
			Inner join A_UserParams up on up.[IdUser] = o.[OperatorId]
			Inner join A_Users u on u.[Id] = o.[OperatorId]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
			and (up.[IsOperator] = 1)	
				
	-- Все супервизоры (должны иметь также доступ к проектам соответствующим)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], st.[groupId], 7
		From A_TaskManager_TaskSupervisors s
			Inner join dbo.A_Users u on u.[id] = s.[SupervisorId]
			Inner join A_Policy_SubjectTypes st on st.[id] = 7
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
			
	-- Супервизоры задачи (должны иметь также доступ к проекту и задаче соответствующим)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], t.[Id], 8
		From A_TaskManager_Tasks t
			Inner join A_TaskManager_TaskSupervisors s on s.[TaskId] = t.[Id]
			Inner join A_Users u on u.[Id] = s.[SupervisorId]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
			
	-- Супервизоры проекта (должны иметь также доступ к проекту и задачам соответствующим)
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select Distinct u.[Id], p.[Id], 9
		From A_TaskManager_Projects p
			Inner join A_TaskManager_Tasks t on t.[IdProject] = p.[Id]
			Inner join A_TaskManager_TaskSupervisors s on s.[TaskId] = t.[Id]
			Inner join A_Users u on u.[Id] = s.[SupervisorId]
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
		
	-- Операторы текущего проекта (копия с другим индексом и размещением ид проекта в subjExtraId)
	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#debug_12'))
		Drop table #debug_12
		
	Select [idUser] as a, st.[groupId] as b, 12 as c, [idSubject] as d 
	Into #debug_12
	From #userroles
		Inner join A_Policy_SubjectTypes st on st.[id] = 12
	Where [subjType] = 6
		
	Insert into #userroles ( [idUser], [idSubject], [subjType], [subjExtraId] )
		Select a, b, c, d
		From #debug_12
			
	-- Супервизоры текущего проекта (копия с другим индексом и размещением ид проекта в subjExtraId)
	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#debug_13'))
		Drop table #debug_13
	
	Select [idUser] as a, st.[groupId] as b, 13 as c, [idSubject] as d
	Into #debug_13
	From #userroles
		Inner join A_Policy_SubjectTypes st on st.[id] = 13
	Where [subjType] = 9
		
	Insert into #userroles ( [idUser], [idSubject], [subjType], [subjExtraId] )
		Select a, b, c, d
		From #debug_13
						
	-- Все пользователи	
	Insert into #userroles ([idUser], [idSubject], [subjType])
		Select u.[Id], st.[groupId], 14
		From A_Users u
			Inner join A_Policy_SubjectTypes st on st.[id] = 14
		Where (u.[IsDeleted] = 0)
			and ((@myuser is NULL) or (@myuser = u.[Id]))
		
	-- Копия привилегии другого объекта (101)
	-- Строится отдельно по запросу
	
		
	-- Вложенные привилегии
	if ((@buildnesting = 1) and (@nestinglevels > 0)) begin
		
		-- Те объекты, чьи привилегии участвуют в замыкании ролевой политики
		--  Мы можем существенно сократить объем итоговой таблицы, поскольку
		--  применение привилегий через привилегии - еще под вопросом.
		if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#closureobjects' ))
			Drop table #closureobjects
		
		Create table #closureobjects ( idobject uniqueidentifier, idprivilege uniqueidentifier )
		
		Insert into #closureobjects ( idobject, idprivilege )
			Select Distinct idSubject, subjExtraId
			From dbo.A_Policy_SubjectPrivileges
			Where subjType = 101
			
		if ( @@rowcount > 0 ) begin
			
			-- Вспомогательная таблица для промежуточного итога группировки при построении списка субъектов из привилегий других объектов	
			if exists ( select * from tempdb..sysobjects where id= object_id('tempdb..#A'))
				Drop table #A
				
			Create table #A ( [idUser] uniqueidentifier, [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [subjType] int, [value] int )
		
			-- Нулевой уровень вложенных ссылок на привилегии объектов (например все читатели проекта X)
			--  С учетом запретов и приоритетов разных субъектных групповых объединений
			-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
			Insert into #A ( [idUser], [idObject], [idPrivilege], [subjType], [value] )
				Select ur.[idUser], sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] ) as [value]
				From A_Policy_SubjectPrivileges sp
					Inner join #userroles ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType])
					Inner join #closureobjects cl on (cl.[idObject] = sp.[idObject]) and (cl.[idPrivilege] = sp.[idPrivilege])
				Group by ur.[idUser], sp.[idObject], sp.[idPrivilege], ur.[subjType]
				
			-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
			--  На выход нас интересуют разрешенные - те у кого value получился 1
			Insert into #userroles ([idUser], [idSubject], [subjType], [subjExtraid], [levelNesting])
				Select a.[idUser], a.[idObject], 101, a.[idPrivilege], 0
				From ( Select a.[idUser], a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
						From #A a
							inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
						Group by a.[idUser], a.[idObject], a.[idPrivilege]
					)x
					Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
					Inner join #A a on (b.[idType] = a.[subjType]) 
										and (x.[idUser] = a.[idUser]) 
										and (x.[idObject] = a.[idObject]) 
										and (x.[idPrivilege] = a.[idPrivilege])
				Where a.[value] = 1
				
					
			-- Остальные уровни вложенных ссылок на привилегии объектов
			Declare @i int
			Set @i = 0
			
			While ( @@rowcount > 0 ) begin
							
				Set @i = @i + 1
				
				if ( @i >= @nestinglevels )
					break
							
				-- Только один следующий уровень участвует
				-- Среди односортных типов ищем разрешения (запреты менее приоритетны)
				Truncate table #A
				
				Insert into #A ( [idUser], [idObject], [idPrivilege], [subjType], [value] )
					Select ur.[idUser], sp.[idObject], sp.[idPrivilege], ur.[subjType], Max ( sp.[value] )
					From A_Policy_SubjectPrivileges sp
						Inner join ( Select * 
									 From #userroles ur
									 Where (ur.[subjType] = 101) and (ur.[levelNesting] = @i-1)
									) ur on (ur.[idSubject] = sp.[idSubject]) and (ur.[subjType] = sp.[subjType]) and (ur.[subjExtraId] = sp.[subjExtraId])
						Inner join #closureobjects cl on (cl.[idObject] = sp.[idObject]) and (cl.[idPrivilege] = sp.[idPrivilege])
					Group by ur.[idUser], sp.[idObject], sp.[idPrivilege], ur.[subjType]
				
				-- Среди всех групп выбираем первую по сортировочному приоритету и именно ее значение value берем
				--  На выход нас интересуют разрешенные - те у кого value получился 1
				if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#debug_nest'))
					Drop table #debug_nest
					
				Select a.[idUser] as a, a.[idObject] as b, 101 as c, a.[idPrivilege] as d, @i as e
				Into #debug_nest
				From ( Select a.[idUser], a.[idObject], a.[idPrivilege], Min(b.[priority]) as priority
						From #A a
							inner join dbo.A_Policy_SubjectTypePriority b on b.[idType] = a.[subjType]
						Group by a.[idUser], a.[idObject], a.[idPrivilege]
					)x
					Inner join dbo.A_Policy_SubjectTypePriority b on b.[priority] = x.[priority]
					Inner join #A a on (b.[idType] = a.[subjType]) 
									and (x.[idUser] = a.[idUser]) 
									and (x.[idObject] = a.[idObject]) 
									and (x.[idPrivilege] = a.[idPrivilege])
				Where (a.[value] = 1)
					and not exists ( Select ur.[idSubject]
									 From #userroles ur 
									 Where (ur.[idSubject] = a.[idObject]) 
										and (ur.[subjExtraId] = a.[idPrivilege]))
										
				Insert into #userroles ([idUser], [idSubject], [subjType], [subjExtraid], [levelNesting])
					Select a, b, c, d, e 
					From #debug_nest
			end
			
			Drop table #A
			
		end
		
		Drop table #closureobjects
	end
	
	if ( @fillcache = 1 ) begin
	
		Insert into A_Policy_SubjectGroupCache ( [idUser], [idSubject], [subjType], [subjExtraId], [levelNesting] )		
			Select [idUser], [idSubject], [subjType], [subjExtraId], [levelNesting]
			From #userroles ur
			
	end else begin
	
			Select [idUser], [idSubject], [subjType], [subjExtraId], [levelNesting]
			From #userroles ur
			Order by ur.[subjType], ur.[levelNesting]		
	end	
		
end
GO