﻿---------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserControls_Save' ))
	Drop procedure dbo.A_UserControls_Save
GO

Create procedure [dbo].[A_UserControls_Save]
	@mode int, -- 1 - by UserA, 0 - by UserB
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	-- Связки новые сохраняющиеся
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( 
		a uniqueidentifier, 
		b uniqueidentifier 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *	
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb'
		)
		
	
	-- На случай удаления всех связок - тех кто пустые 
	If object_id('tempdb..#tmp_xml2') is not NULL
		Drop table #tmp_xml2	
	Create table #tmp_xml2 ( a uniqueidentifier )
		
	Declare @idoc2 int
	Exec sp_xml_preparedocument @idoc2 OUTPUT, @xml
	
	Insert into #tmp_xml2
		Select *
		From OpenXml (@idoc, '/items/a', 2)	
		With ( ida varchar(50) '@ida' )		
		
		
	-- Операции изменения
	if ( @mode = 1 ) begin
		
		Delete
		From dbo.A_UserControls
		Where exists ( Select 1 From #tmp_xml2 x Where dbo.A_UserControls.UserA = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControls.UserA = x.a) and (A_UserControls.UserB = x.b) )
			
		Insert into dbo.A_UserControls (UserA, UserB)
			Select Distinct a, b
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControls u Where (u.UserA = x.a) and (u.UserB = x.b) )
			
	end else begin
			
		Delete
		From dbo.A_UserControls
		Where exists ( Select 1 From #tmp_xml2 x Where dbo.A_UserControls.UserB = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControls.UserB = x.a) and (A_UserControls.UserA = x.b) )
			
		Insert into dbo.A_UserControls (UserA, UserB)
			Select Distinct b, a
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControls u Where (u.UserA = x.b) and (u.UserB = x.a) )
	
	end
	
end
GO

---------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserOutcomingLines_Save' ))
	Drop procedure dbo.A_UserOutcomingLines_Save
GO

Create procedure [dbo].[A_UserOutcomingLines_Save]
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( 
		a uniqueidentifier, 
		b uniqueidentifier, 
		idx int 
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb',
			idx int '@idx'
		)	
		

	-- Удаляем те, которых не осталось
	Delete
	From dbo.A_UserOutcomingLines
	Where exists ( Select 1 From #tmp_xml2 x Where dbo.A_UserOutcomingLines.IdUser = x.a )			
		and not exists( Select 1 
						From #tmp_xml x 
						Where (dbo.A_UserOutcomingLines.IdUser = x.a) 
							and (dbo.A_UserOutcomingLines.IdLine = x.b))
	
	-- Обновляем индексы у тех что были и остались 
	Update w
	Set w.LineIndex = x.idx
	From dbo.A_UserOutcomingLines w, #tmp_xml x
	Where (w.IdUser = x.IdUser)
		and (w.IdLine = x.IdLine)
		and (w.LineIndex != x.idx)
		
	-- Добавляем новые
	Insert into dbo.A_UserOutcomingLines (IdUser, IdLine, LineIndex)
		Select Distinct a, b, idx
		From #tmp_xml x
		Where not exists ( Select 1 
							From dbo.A_UserOutcomingLines w
							Where (w.IdUser = x.a) 
								and (w.IdLine = x.b))
	
end
GO

---------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_UserControlLines_Save' ))
	Drop procedure dbo.A_UserControlLines_Save
GO

Create procedure [dbo].[A_UserControlLines_Save]
	@mode int, -- 1 - by User, 0 - by Line
	@xml ntext, -- <items><a ida=""><b idb=""/><b idb=""/></a><a ida....>
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid

	
	-- Связки новые или существующие
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml ( a uniqueidentifier, b uniqueidentifier )
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/a/b', 2)	
		With ( 
			ida varchar(50) '../@ida',
			idb varchar(50) '@idb'
		)	
		
	-- На случай удаления всех связок - тех кто пустые ноды
	If object_id('tempdb..#tmp_xml2') is not NULL
		Drop table #tmp_xml2	
	Create table #tmp_xml2 ( a uniqueidentifier )
		
	Declare @idoc2 int
	Exec sp_xml_preparedocument @idoc2 OUTPUT, @xml
	
	Insert into #tmp_xml2
		Select *
		From OpenXml (@idoc, '/items/a', 2)	
		With ( ida varchar(50) '@ida' )		
		
		
	if ( @mode = 1 ) begin
		
		Delete
		From dbo.A_UserControlLines
		Where exists ( Select 1 From #tmp_xml2 x Where dbo.A_UserControlLines.UserA = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControlLines.UserA = x.a) and (A_UserControlLines.IdLine = x.b) )
			
		Insert into dbo.A_UserControlLines (UserA, IdLine)
			Select Distinct a, b
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControlLines u Where (u.UserA = x.a) and (u.IdLine = x.b) )
			
	end else begin
	
		Delete
		From dbo.A_UserControlLines
		Where exists ( Select 1 From #tmp_xml2 x Where dbo.A_UserControlLines.IdLine = x.a )
			and not exists ( Select 1 From #tmp_xml x Where (A_UserControlLines.IdLine = x.a) and (A_UserControlLines.UserA = x.b) )
			
		Insert into dbo.A_UserControlLines (UserA, IdLine)
			Select Distinct b, a
			From #tmp_xml x
			Where not exists ( Select 1 From dbo.A_UserControlLines u Where (u.UserA = x.b) and (u.IdLine = x.a) )
	
	end
	
end
GO

---------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_Workplaces_Save' ))
	Drop procedure dbo.A_Admin_OfficeMap_Workplaces_Save
GO

Create procedure [dbo].[A_Admin_OfficeMap_Workplaces_Save]
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Рабочие места (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	(
		id uniqueidentifier primary key, 
		type int, 
		parentid uniqueidentifier, 
		address nvarchar(500) collate database_default, 
		hdb bit, 
		hfe bit, 
		hl bit, 
		hel bit, 
		hil bit, 
		hgsm bit, 
		name nvarchar(500) collate database_default, 
		lineid uniqueidentifier, 
		sort int
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc, '/items/wp', 2)	
		With ( 
			id varchar(50) '@id',
			type int '@type',
			parentid varchar(50) '@par',
			address nvarchar(500) '@addr',
			hdb bit '@hdb',
			hfe bit '@hfe',
			hl bit '@hl',
			hel bit '@hel',
			hil bit '@hil',
			hgsm bit '@hgsm',
			name nvarchar(500) '@name',
			lineid varchar(50) '@lid',
			sort int '@sort'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_Workplaces
	Where not exists ( Select 1 From #tmp_xml x Where A_Workplaces.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.ParentId = x.parentid,
		w.Address = x.address,
		w.HasDatabase = x.hdb,
		w.HasExtLines = x.hel,
		w.HasIntLines = x.hil,
		w.HasGsm = x.hgsm,
		w.HasFastEngine = x.hfe,
		w.HasLogic = x.hl,
		w.Name = x.name,
		w.LineId = x.lineid,
		w.SortIndex = x.sort		
	From dbo.A_Workplaces w, #tmp_xml x
	Where (w.Id = x.id)
		and ((w.Type != x.type)
			or (w.ParentId != x.parentid)
			or (w.Address != x.address)
			or (w.HasDatabase != x.hdb)
			or (w.HasExtLines != x.hel)
			or (w.HasIntLines != x.hil)
			or (w.HasGsm != x.hgsm)
			or (w.HasFastEngine != x.hfe)
			or (w.HasLogic != x.hl)
			or (w.Name != x.name)
			or (w.LineId != x.lineid)
			or (w.SortIndex != x.sort))
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_Workplaces (Id, Type, ParentId, Address, HasDatabase, HasExtLines, HasIntLines, HasGsm, HasFastEngine, HasLogic, Name, LineId, SortIndex)
		Select Distinct id, type, parentid, address, hdb, hfe, hl, hel, hil, hgsm, name, lineid, sort
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_Workplaces w Where w.Id = x.id )
			
end
GO


---------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Rule_Records_Save' ))
--	Drop procedure dbo.A_Admin_Rule_Records_Save
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Rule_Save' ))
--	Drop procedure dbo.A_Admin_Rule_Save
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Rule_SaveFull' ))
	Drop procedure dbo.A_Admin_Rule_SaveFull
GO

Create Procedure [dbo].[A_Admin_Rule_SaveFull]
	@id uniqueidentifier,
	@name nvarchar ( 100 ), 
	@priority int,
	@type int = 0,
	@xmlrecs ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
  -- Сохраняем правило
	if exists ( Select 1 From dbo.A_Rules where ID = @id ) begin
	
		Update A_Rules 
		Set Name = @name, 
			Type = @type,
			Priority = @priority
		Where (ID = @id)
			and ((Name != @name)
				or (Type != @type)
				or (Priority != @priority))
		
	end else begin
	
		Insert 
		Into A_Rules ( ID, Name, Type, Priority ) 
		Values ( @id, @name, @type, @priority )
		
	end
	
  -- Сохраняем записи правила
	if ( @xmlrecs is NULL)
		return
		
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		id uniqueidentifier,
		addrtype int,
		starttime int,
		finishtime int,
		reactid uniqueidentifier,
		account nvarchar(100) collate database_default,
		caption nvarchar(255) collate database_default
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlrecs
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc1, '/items/rr', 2)	
		With ( 
			id varchar(50) '@id',
			addrtype int '@adt',
			starttime int '@st',
			finishtime int '@ft',
			reactid varchar(50) '@rcid',
			account nvarchar(100) '@acc',
			caption nvarchar(255) '@cap'
		)
	
	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	Delete 
	From dbo.A_RuleRecords
	Where (RuleId = @id)
		and not exists( Select 1
						From #tmp_xml x
						Where dbo.A_RuleRecords.Id = x.id )
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.InnerAddressType = x.addrtype,
		w.StartTime = x.starttime,
		w.FinishTime = x.finishtime,
		w.ReactID = x.reactid,
		w.Account = x.account
	From dbo.A_RuleRecords w, #tmp_xml x
	Where (w.RuleId = @id)
		and (w.Id = x.id)
		and ((w.InnerAddressType != x.addrtype)
			or (w.StartTime != x.starttime)
			or (w.FinishTime != x.finishtime)
			or (w.ReactID != x.reactid)
			or (w.Account != x.account))
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_RuleRecords ( Id, InnerAddressType, StartTime, FinishTime, RuleID, ReactID, Account )
		Select x.id, x.addrtype, x.starttime, x.finishtime, @id, x.reactid, x.account
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_RuleRecords w
							Where (w.RuleID = @id)
								and (w.Id = x.id))	

end
GO

---------------------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Number_Action_Save' ))
--	Drop procedure dbo.A_Admin_Number_Action_Save
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Number_Save' ))
--	Drop procedure dbo.A_Admin_Number_Save
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_Number_SaveFull' ))
	Drop procedure dbo.A_Admin_Number_SaveFull
GO

Create Procedure [dbo].[A_Admin_Number_SaveFull]
	@id uniqueidentifier,
	@prefix nvarchar ( 20 ),
	@visible bit,
	@description nvarchar ( 100 ),
	@xmlrecs ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	

  -- Сохраняем номер
	if exists ( Select 1 From dbo.A_NumberPlan where [Id] = @id ) begin
	
		Update A_NumberPlan 
		Set [Id] = @id,	
			[Prefix] = @prefix,
			[Visible] = @visible,
			[Description] = @description
		Where ([Id] = @id)
			and (([Prefix] != @prefix)
				or ([Visible] != @visible)
				or ([Description] != @description))
		
	end else begin
	
		Insert 
		Into A_NumberPlan ( [Id], [Prefix], [Visible], [Description] ) 
		Values ( @id, @prefix, @visible, @description )
		
	end
	
   -- Сохраняем записи номера
	if ( @xmlrecs is NULL)
		return
		
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		id uniqueidentifier,
		numtype int,
		extratype int,
		extraid uniqueidentifier,
		extrastr nvarchar(500) collate database_default,
		priority int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlrecs
	
	Insert into #tmp_xml
		Select *
		From OpenXml (@idoc1, '/items/rr', 2)	
		With ( 
			id varchar(50) '@id',
			numtype int '@type',
			extratype int '@sptype',
			extraid varchar(50) '@rcid',
			extrastr nvarchar(500) '@ad',
			priority int '@prior'
		)
	

	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	Delete 
	From dbo.A_NumberPlanAction
	Where (NumId = @id)
		and not exists( Select 1
						From #tmp_xml x
						Where dbo.A_NumberPlanAction.Id = x.id )
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.NumType = x.numtype,
		w.ExtraType = x.extratype,
		w.ExtraId = x.extraid,
		w.ExtraStr = x.extrastr,
		w.Priority = x.priority
	From dbo.A_NumberPlanAction w, #tmp_xml x
	Where (w.NumId = @id)
		and (w.Id = x.id)
		and ((w.NumType != x.numtype)
			or (w.NumType != x.numtype)
			or (w.ExtraType != x.extratype)
			or (w.ExtraId != x.extraid)
			or (w.ExtraStr != x.extrastr)
			or (w.Priority != x.priority))
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_NumberPlanAction ( Id, NumId, NumType, ExtraType, ExtraId, ExtraStr, Priority, AllowShare )
		Select x.id, @id, x.numtype, x.extratype, x.extraid, x.extrastr, x.priority, 1
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_NumberPlanAction w
							Where (w.NumID = @id)
								and (w.Id = x.id))	
	
end
GO