﻿---------------------------------------------------------------------------------
if exists ( Select * From sysobjects where id=object_id ( 'dbo.A_UserRoles_SaveRole' ))
	Drop procedure dbo.A_UserRoles_SaveRole
GO

Create procedure [dbo].[A_UserRoles_SaveRole]
	@id uniqueidentifier,
	@name nvarchar(200),
	@baserole uniqueidentifier = NULL,
	@domination bit,
	@isbasic bit = 0,
 	@xmlrights ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Сохраняем саму роль
	if exists ( Select 1
				From dbo.A_UserRoles_Roles
				Where Id = @id ) 
	begin
				
		Update dbo.A_UserRoles_Roles
		Set 
			Id = @id,
			Name = @name,
			BaseRole = @baserole,
			Domination = @domination
		Where (Id = @id)
			and ((Name != @name)
				or (BaseRole != @baserole)
				or (Domination != @domination))

	end else begin
	
		-- Добавление новой
		Insert into dbo.A_UserRoles_Roles ( Id, Name, BaseRole, Domination ) Values ( @id, @name, @baserole, @domination )
		
		-- Заказ на пересчет кэша политики доступа
		exec A_Policy_SubjectGroupCache_SetUpdate
		
	end
	
		
	-- Обновление привилегий у роли
	if ( @xmlrights is NULL)
		return
		
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		idright uniqueidentifier,
		value int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlrights
	
	Insert into #tmp_xml (idright, value)
		Select idright, value
		From OpenXml (@idoc1, '/items/a', 2)	
		With ( 
			idright varchar(50) '@id',
			value int '@val'			
		)
	
	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	if ( @isbasic = 1 ) begin
		
		-- Базовая роль сохраняется базово при старте - нельзя трогать кастомные привилегии
		Delete 
		From dbo.A_UserRoles_RightsOfRoles
		Where (IdRole = @id)
			and not exists( Select 1
							From #tmp_xml x
							Where dbo.A_UserRoles_RightsOfRoles.IdRight = x.idright )
			and not exists( Select 1
							From dbo.A_UserRoles_Rights r
							Where (dbo.A_UserRoles_RightsOfRoles.IdRight = r.Id)
								and r.IsBasic = 0)
	end else begin
	
		-- Любая роль сохраняется из клиента - участвуют уже все привилегии, в т.ч. кастомные
		Delete 
		From dbo.A_UserRoles_RightsOfRoles
		Where (IdRole = @id)
			and not exists( Select 1
							From #tmp_xml x
							Where dbo.A_UserRoles_RightsOfRoles.IdRight = x.idright )
	end
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.Value = x.value
	From dbo.A_UserRoles_RightsOfRoles w, #tmp_xml x
	Where (w.IdRole = @id)
		and (w.IdRight = x.idright)
		and (w.Value != x.Value)
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_UserRoles_RightsOfRoles ( IdRole, IdRight, Value )
		Select @id, x.idright, x.value
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_UserRoles_RightsOfRoles w
							Where (w.IdRole = @id)
								and (w.IdRight = x.idright))
		
end
GO


-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_PlugIns_Save'))
	Drop procedure dbo.[A_PlugIns_Save]
GO

Create procedure [dbo].[A_PlugIns_Save]
	@id uniqueidentifier,
	@type int,
	@objectname nvarchar ( 400 ) = null,
	@name nvarchar ( 400 ),
	@description nvarchar ( 4000 ) = null,
	@path nvarchar ( 1000 ) = null,
	@loadtype int = 1,
	@loadmoment int = 0,
	@destination int = 1,
	@logmode int = 0,
	@authorizeduserid uniqueidentifier = NULL	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Update A_PlugIns
	Set 	Type = @type,
		ObjectName = @objectname,
		Name = @name,
		Description = @description,
		Path = @path,
		LoadType = @loadtype,
		LoadMoment = @loadmoment,
		Destination = @destination,
		LogMode = @logmode
	Where Id = @id

	if ( @@rowcount = 0 )
		Insert 
		Into A_PlugIns ( Id, Type, ObjectName, Name, Description, Path, LoadType, LoadMoment, Destination, LogMode )
		Values ( @id, @type, @objectname, @name, @description, @path, @loadtype, @loadmoment, @destination, @logmode )
end
GO