﻿use oktell_settings
GO

if not exists ( select * from sysobjects where id = object_id ( 'dbo.A_Scripts' )) begin	
	Create table dbo.A_Scripts
	(
		[id] uniqueidentifier not null primary key,
		[name] nvarchar(1000),
		[type] int,
		[idproject] uniqueidentifier
	)
end
GO

if not exists ( Select 1 from sysindexes where id = object_id ( 'dbo.A_Scripts' ) and name like 'idx_Scripts_Project' )
	Create index idx_Scripts_Project on dbo.A_Scripts ( idproject)
GO

use oktell
GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Scripts' ))
	Drop view dbo.A_Scripts
GO

Create view dbo.A_Scripts
as
	Select *
	From oktell_settings.dbo.A_Scripts
GO

----------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Scripts_Save'))
	Drop procedure dbo.A_Scripts_Save
GO

Create procedure dbo.A_Scripts_Save
	@id uniqueidentifier = NULL,
	@name nvarchar(1000),
	@type int,
	@idproject uniqueidentifier = NULL
as
begin
	
	if ( @id is NULL ) begin
	
		if not exists ( Select 1 
						From dbo.A_Scripts 
						Where ([type] = @type)
							and ([idproject] = @idproject)
							and ([name] = @name))
			
			Insert into dbo.A_Scripts ( id, name, type, idproject )
			Values ( NewID(), @name, @type, @idproject )
		
	end	else if exists ( Select 1 From dbo.A_Scripts Where id = @id ) begin
	
		Update dbo.A_Scripts
		Set [name] = @name,
			[type] = @type,
			[idproject] = @idproject
		Where (id = @id)
			and ((name != @name)
				or (type != @type)
				or (idproject != @idproject))
				
	end else begin
			
		Insert into dbo.A_Scripts ( id, name, type, idproject )
		Values ( @id, @name, @type, @idproject )
	
	end
	
end
GO

----------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Scripts_GetAll'))
	Drop procedure dbo.A_Scripts_GetAll
GO

Create procedure dbo.A_Scripts_GetAll
	@all bit = 0,
	@idproject uniqueidentifier = NULL	
as
begin
	
	if ( @all = 1 ) begin
		
		Select [id], [name], [type], [idproject]
		From dbo.A_Scripts
		
	end else begin
	
		Select [id], [name], [type], [idproject]
		From dbo.A_Scripts
		Where idproject = @idproject
	
	end
		
end
GO

----------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Scripts_GetById'))
	Drop procedure dbo.A_Scripts_GetById
GO

Create procedure dbo.A_Scripts_GetById
	@id uniqueidentifier
as
begin
	
	Select [id], [name], [type], [idproject]
	From dbo.A_Scripts
	Where [id] = @id
	
end
GO

---------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Scripts_Remove'))
	Drop procedure dbo.A_Scripts_Remove
GO

Create procedure dbo.A_Scripts_Remove
	@id uniqueidentifier = NULL,
	@name nvarchar(1000) = NULL,
	@type int = NULL,
	@idproject uniqueidentifier = NULL
as
begin

	if ( not @id is NULL ) begin
		
		Delete
		From dbo.A_Scripts
		Where id = @id
		
	end else begin
	
		Delete
		From dbo.A_Scripts
		Where (name = @name)
			and (type = @type)
			and (idproject = @idproject)
			
	end
	
end
GO
