﻿use oktell_settings
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts' ))
	if not exists (Select * FROM syscolumns where id=object_id ( 'dbo.A_Scripts' ) and name like 'modifieduserid' )
		Alter table dbo.A_Scripts Add modifieduserid uniqueidentifier
GO

use oktell
GO

if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts' ))
	Drop view dbo.A_Scripts
GO

Create view A_Scripts
as 
	Select * FROM oktell_settings.dbo.A_Scripts
GO

---------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts_Save' ))
	Drop procedure dbo.A_Scripts_Save
GO

Create procedure [dbo].[A_Scripts_Save]
	@id uniqueidentifier = NULL,
	@name nvarchar(1000),
	@type int,
	@idproject uniqueidentifier = NULL,
	@modifieduserid uniqueidentifier = NULL
as
begin
	
	if ( @id is NULL ) begin
	
		if exists ( Select 1 
						From dbo.A_Scripts 
						Where ([type] = @type)
							and ([idproject] = @idproject)
							and ([name] = @name))
		begin
			
			Select @id = id
			From dbo.A_Scripts
			Where (([type] = @type)
					and ([idproject] = @idproject)
					and ([name] = @name))
					
			Update dbo.A_Scripts
			Set [modifieduserid] = @modifieduserid
			Where (([type] = @type)
					and ([idproject] = @idproject)
					and ([name] = @name))
				and (modifieduserid != @modifieduserid)
					
		end else begin					
			
			Set @id = NewID()
			
			Insert into dbo.A_Scripts ( id, name, type, idproject, modifieduserid )
			Values ( @id, @name, @type, @idproject, @modifieduserid )
			
		end
		
	end	else if exists ( Select 1 From dbo.A_Scripts Where id = @id ) begin
	
		Update dbo.A_Scripts
		Set [name] = @name,
			[type] = @type,
			[idproject] = @idproject,
			[modifieduserid] = @modifieduserid
		Where (id = @id)
			and ((name != @name)
				or (type != @type)
				or (idproject != @idproject)
				or (modifieduserid != @modifieduserid))
				
	end else begin
			
		Insert into dbo.A_Scripts ( id, name, type, idproject )
		Values ( @id, @name, @type, @idproject )
	
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	if (@modifieduserid = '00000000-0000-0000-0000-000000000000') 
		Set @modifieduserid = NULL
		
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 106,
		@name = @name,
		@idParentProject = @idproject,
		@actionUserId = @modifieduserid
	
end
GO

---------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts_GetAll' ))
	Drop procedure dbo.A_Scripts_GetAll
GO

Create procedure [dbo].[A_Scripts_GetAll]
	@all bit = 0,
	@idproject uniqueidentifier = NULL	
as
begin
	
	if ( @all = 1 ) begin
		
		Select [id], [name], [type], [idproject], [modifieduserid]
		From dbo.A_Scripts
		
	end else begin
	
		Select [id], [name], [type], [idproject], [modifieduserid]
		From dbo.A_Scripts
		Where idproject = @idproject
	
	end
		
end
GO

-----------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts_GetById' ))
	Drop procedure dbo.A_Scripts_GetById
GO

Create procedure [dbo].[A_Scripts_GetById]
	@id uniqueidentifier
as
begin
	
	Select [id], [name], [type], [idproject], [modifieduserid]
	From dbo.A_Scripts
	Where [id] = @id
	
end
GO

-----------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Scripts_Remove' ))
	Drop procedure dbo.A_Scripts_Remove
GO

Create procedure [dbo].[A_Scripts_Remove]
	@id uniqueidentifier = NULL,
	@name nvarchar(1000) = NULL,
	@type int = NULL,
	@idproject uniqueidentifier = NULL
as
begin

	if ( not @id is NULL ) begin
		
		Delete
		From dbo.A_Scripts
		Where id = @id
		
	end else begin
	
		Select @id = id
		From dbo.A_Scripts
		Where (name = @name)
			and (type = @type)
			and (idproject = @idproject)
			
		Delete
		From dbo.A_Scripts
		Where (name = @name)
			and (type = @type)
			and (idproject = @idproject)
			
	end
	
	-- Из таблиц политики прав доступа к объектам
	exec A_Policy_Objects_Remove @id		
	-- Заказ на очистку таблиц политики доступа
	exec A_Policy_SetFilter
	
end
GO

--------------------------------------------------------------------
--------------------------------------------------------------------
--------------------------------------------------------------------
Update dbo.A_Policy_PrivilegeDefaults
Set value = 1
Where (idprivilege in ('8BBFAB7B-460C-4D5B-B4E4-1695F86AF6D1', 'B8CDCA65-866D-4636-92B9-8C8F58FF76D3'))
	and (subjType = 14)
GO

--------------------------------------------------------------------
Declare CUR Cursor For
	Select id, name, idproject
	From A_Scripts

Open CUR

Declare @id uniqueidentifier, @name nvarchar(500), @idproject uniqueidentifier
Fetch Next From CUR Into @id, @name, @idproject

While (@@fetch_status=0) begin
	
	exec A_Policy_Objects_Save
			@idObject = @id,
			@type = 106,
			@name = @name,
			@idParentProject = @idproject,
			@actionUserId = NULL
		
	Fetch Next From CUR Into @id, @name, @idproject
end	

Close CUR
Deallocate CUR
GO
	
	
	