﻿
use oktell_settings
GO

if (not object_id('dbo.A_UserRoles_Roles') is null) 
	if not exists (Select * From syscolumns where (id = object_id ( 'dbo.A_UserRoles_Roles' )) 
										and (name like 'IsBasic'))
		Alter Table dbo.A_UserRoles_Roles Add IsBasic bit not null Default 0
GO


Update dbo.A_UserRoles_Roles
Set IsBasic = 0
Where IsBasic is null
GO

use oktell
GO

if (not object_id ( 'dbo.A_UserRoles_Roles' ) is null)
	Drop view dbo.A_UserRoles_Roles
GO

Create view dbo.A_UserRoles_Roles
as
	Select *
	From oktell_settings.dbo.A_UserRoles_Roles
GO


if (not object_id('dbo.A_UserRoles_Load_Roles') is null)
	Drop procedure dbo.A_UserRoles_Load_Roles
GO

Create Procedure [dbo].[A_UserRoles_Load_Roles]
	@onlyforuserid uniqueidentifier = null,
	@onlyforroleid uniqueidentifier = null,
	@skipdominant bit = 0,
	@skipbasic bit = 0
As
Begin

    if ( @onlyforuserid is null ) begin

		Select Id, Name, BaseRole, Domination, IsNull(UserCount, 0) as UserCount, 0 as levelNesting, IsBasic
		From dbo.A_UserRoles_Roles a
			left join ( Select IdRole, UserCount = Count ( * )
						From A_UserRoles_RolesOfUsers
						Group By IdRole
			) b On a.Id = b.IdRole
		Where (( @skipdominant = 0 ) or ( Domination != 1 ))
			And (( @skipbasic = 0 ) or ( IsBasic != 1 ))
			And ((@onlyforroleid is null) or ( a.Id = @onlyforroleid ))
		Order by Name

    end else begin

		Declare @roles table (id uniqueidentifier, levelNesting int)
		Insert into @roles ( id, levelNesting )
			Select IdRole, 0
			From A_UserRoles_RolesOfUsers
			Where IdUser = @onlyforuserid

		Declare @i int
		Set @i = 0			
		while ( @@rowcount > 0 ) begin
			Set @i = @i + 1
			if ( @i > 20 )
				break
				
			Insert into @roles ( id, levelNesting )
				Select rx.Id, @i
				From A_UserRoles_Roles rs
					Inner join @roles r on r.Id = rs.Id
					Inner join A_UserRoles_Roles rx on rx.Id = rs.BaseRole
				Where (r.levelNesting = @i - 1)
					and not exists ( Select id from @roles x where x.Id = rx.Id )
		end		

		Select a.Id, a.Name, a.BaseRole, a.Domination, IsNull(b.UserCount, 0) as UserCount, r.levelNesting, IsBasic
		From dbo.A_UserRoles_Roles a
			inner join @roles r on a.Id = r.id
	    	left join ( Select IdRole, UserCount = Count ( * )
						From A_UserRoles_RolesOfUsers
						Group By IdRole 
			) b On a.Id = b.IdRole
		Where (( @skipdominant = 0 ) or ( a.Domination != 1 ))
			And (( @skipbasic = 0 ) or ( IsBasic != 1 ))
			And ((@onlyforroleid is null) or ( a.Id = @onlyforroleid ))
		Order by Name

    end

End
GO


if (not object_id('dbo.A_UserRoles_SaveRole') is null)
	Drop procedure dbo.A_UserRoles_SaveRole
GO

Create procedure [dbo].[A_UserRoles_SaveRole]
	@id uniqueidentifier,
	@name nvarchar(200),
	@baserole uniqueidentifier = NULL,
	@domination bit,
	@isbasic bit = 0,
 	@xmlrights ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Сохраняем саму роль
	if exists ( Select 1
				From dbo.A_UserRoles_Roles
				Where Id = @id ) 
	begin
				
		Update dbo.A_UserRoles_Roles
		Set 
			Id = @id,
			Name = @name,
			BaseRole = @baserole,
			Domination = @domination,
			IsBasic = @isbasic
		Where (Id = @id)
			and ((Name != @name)
				or ((BaseRole is null) or (@baserole is null) or (BaseRole != @baserole))
				or (Domination != @domination))
				
		Select cast(0 as int) as created

	end else begin
	
		-- Добавление новой
		Insert into dbo.A_UserRoles_Roles ( Id, Name, BaseRole, Domination, IsBasic ) 
		Values ( @id, @name, @baserole, @domination, @isbasic )
		
		-- Заказ на пересчет кэша политики доступа
		exec A_Policy_SubjectGroupCache_SetUpdate
		
		-- 
		Select cast(1 as int) as created
		
	end
	
		
	-- Обновление привилегий у роли
	if ( @xmlrights is NULL)
		return
		
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		idright uniqueidentifier,
		value int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmlrights
	
	Insert into #tmp_xml (idright, value)
		Select idright, value
		From OpenXml (@idoc1, '/items/a', 2)	
		With ( 
			idright varchar(50) '@id',
			value int '@val'			
		)
	
	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	if ( @isbasic = 1 ) begin
		
		-- Базовая роль сохраняется базово при старте - нельзя трогать кастомные привилегии
		Delete 
		From dbo.A_UserRoles_RightsOfRoles
		Where (IdRole = @id)
			and not exists( Select 1
							From #tmp_xml x
							Where dbo.A_UserRoles_RightsOfRoles.IdRight = x.idright )
			and not exists( Select 1
							From dbo.A_UserRoles_Rights r
							Where (dbo.A_UserRoles_RightsOfRoles.IdRight = r.Id)
								and r.IsBasic = 0)
	end else begin
	
		-- Любая роль сохраняется из клиента - участвуют уже все привилегии, в т.ч. кастомные
		Delete 
		From dbo.A_UserRoles_RightsOfRoles
		Where (IdRole = @id)
			and not exists( Select 1
							From #tmp_xml x
							Where dbo.A_UserRoles_RightsOfRoles.IdRight = x.idright )
	end
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.Value = x.value
	From dbo.A_UserRoles_RightsOfRoles w, #tmp_xml x
	Where (w.IdRole = @id)
		and (w.IdRight = x.idright)
		and (w.Value != x.Value)
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_UserRoles_RightsOfRoles ( IdRole, IdRight, Value )
		Select @id, x.idright, x.value
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_UserRoles_RightsOfRoles w
							Where (w.IdRole = @id)
								and (w.IdRight = x.idright))
		
end
GO


if (not object_id('dbo.A_UserRoles_CopyCustomRoleRights') is null)
	Drop procedure dbo.A_UserRoles_CopyCustomRoleRights
GO

Create procedure [dbo].[A_UserRoles_CopyCustomRoleRights]
	@idfrom uniqueidentifier,
	@idto uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
As
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Insert into A_UserRoles_RightsOfRoles (IdRole, IdRight, Value)
		Select @idto, rr.IdRight, rr.Value
		From A_UserRoles_RightsOfRoles rr
			Inner join A_UserRoles_Rights r on r.id = rr.idright
		Where (rr.IdRole = @idfrom)
			and (r.IsBasic = 0)
			and not exists (Select 1 From A_UserRoles_RightsOfRoles rx Where rx.IdRole = @idto and rx.IdRight = rr.idright)
		
end
GO

-------------------------------


if (not object_id('dbo.A_Policy_SubjectGroups_GetList') is null)
	Drop procedure dbo.A_Policy_SubjectGroups_GetList
GO

Create procedure [dbo].[A_Policy_SubjectGroups_GetList]
	@objecttype int = 0
as 
begin

	-- Возвращает список всех именованных субъектов, доступных для назначения прав
	
	
	-- Нужно ли проектные субъекты
	Declare @useproject bit
	Select @useproject = useProject
	From dbo.A_Policy_ObjectTypes
	Where id = @objecttype
	if ( @useproject is NULL )
		Set @useproject = case when ( @objecttype = 0 ) then 1 else 0 end
		
	
	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#subjects'))
		Drop table #subjects
		
	Create table #subjects ( [type] int, [name] nvarchar(500), [id] uniqueidentifier, [extraId] uniqueidentifier, [equalMode] int )
			


	-- Пользователи
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 1, [Name], [Id], null, 2
		From A_Users u
		Where u.IsDeleted = 0
	
	-- Отделы
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 2, [Name], [Id], null, 2
		From A_Groups g	
	
	-- Роли
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 3, r.[Name], r.[Id], null, 2
		From A_UserRoles_Roles r
		Where not exists ( Select Id from A_Users u Where u.Id = r.Id )
			and (Domination = 0) 
			and (IsBasic = 0)
	
	
	-- Все операторы	
	Declare @alloperators nvarchar(200)
	Select @alloperators = langvalue 
	From A_LangTexts
	Where langkey like 'alloperators'
	if ( @alloperators is NULL )
		Set @alloperators = N'Все операторы'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 4, @alloperators, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 4
		
		
	if ( @useproject = 1 ) begin
	
		-- Операторы текущего проекта
		Declare @currentprojectoperators nvarchar(200)
		Select @currentprojectoperators = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectoperators'
		if ( @currentprojectoperators is NULL )
			Set @currentprojectoperators = N'Операторы текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 12, @currentprojectoperators, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 12
	end
	
	
	---- Операторы задачи
	--Declare @taskoperators nvarchar(200)
	--Select @taskoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'taskoperators'
	--if ( @taskoperators is NULL )
	--	Set @taskoperators = 'Операторы задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @taskoperators + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Операторы проекта
	--Declare @projectoperators nvarchar(200)
	--Select @projectoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectoperators'
	--if ( @projectoperators is NULL )
	--	Set @projectoperators = 'Операторы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @projectoperators + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	
	
	-- Все супервизоры
	Declare @allsupervisors nvarchar(200)
	Select @allsupervisors = langvalue 
	From A_LangTexts
	Where langkey like 'allsupervisors'
	if ( @allsupervisors is NULL )
		Set @allsupervisors = N'Все супервизоры'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 7, @allsupervisors, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 7
	
	
	if ( @useproject = 1 ) begin
	
		-- Супервизоры текущего проекта	
		Declare @currentprojectsupervisors nvarchar(200)
		Select @currentprojectsupervisors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectsupervisors'
		if ( @currentprojectsupervisors is NULL )
			Set @currentprojectsupervisors = N'Супервизоры текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 13, @currentprojectsupervisors, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 13	
	end
	
	---- Супервизоры задачи
	--Declare @tasksupervisors nvarchar(200)
	--Select @tasksupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'tasksupervisors'
	--if ( @tasksupervisors is NULL )
	--	Set @tasksupervisors = 'Супервизоры задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 8, @tasksupervisors + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Супервизоры проекта
	--Declare @projectsupervisors nvarchar(200)
	--Select @projectsupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectsupervisors'
	--if ( @projectsupervisors is NULL )
	--	Set @projectsupervisors = 'Супервизоры проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 9, @projectsupervisors + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	

	---- Владельцы проекта
	--Declare @projectowners nvarchar(200)
	--Select @projectowners = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectowners'
	--if ( @projectowners is NULL )
	--	Set @projectowners = 'Владельцы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 10, @projectowners + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p


	-- По другим привилегиями других объектов - ниже
	
	
	-- Все пользователи
	Declare @allusers nvarchar(200)
	Select @allusers = langvalue 
	From A_LangTexts
	Where langkey like 'allusers'
	if ( @allusers is NULL )
		Set @allusers = N'Все пользователи'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 14, @allusers, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 14


	-- По привилегиям всех объектов - слишком много попадет в список. Пропустим пока.
	-- Только на проекты сошлемся
	
	if ( @useproject = 1 ) begin
		
		Declare @idempty uniqueidentifier
		Set @idempty = '00000000-0000-0000-0000-000000000000'
		
		
		-- Читатели текущего проекта
		Declare @currentprojectviewers nvarchar(200)
		Select @currentprojectviewers = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectviewers'
		if ( @currentprojectviewers is NULL )
			Set @currentprojectviewers = N'Пользователи с правом просмотра текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectviewers, @idempty, 'F876E650-7972-475A-AE96-BCCF831AF818', 4
					
						
		-- Редакторы текущего проекта
		Declare @currentprojectredactors nvarchar(200)
		Select @currentprojectredactors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectredactors'
		if ( @currentprojectredactors is NULL )
			Set @currentprojectredactors = N'Пользователи с правом изменения текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectredactors, @idempty, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', 4
						
	end
	
	
	-- Итоговый вывод
	Select s.[type], tp.[priority], s.[name], s.[id], s.[extraId], s.[equalMode]
	From #subjects s
		Inner join dbo.A_Policy_SubjectTypePriority tp on tp.[idType] = s.[type]
	Order By tp.[priority], s.[name]
	
end
GO

if (not object_id('dbo.A_Scripts_Save') is null)
	Drop procedure dbo.[A_Scripts_Save]
GO

Create procedure [dbo].[A_Scripts_Save]
	@id uniqueidentifier = NULL,
	@name nvarchar(1000),
	@type int,
	@idproject uniqueidentifier = NULL,
	@modifieduserid uniqueidentifier = NULL
as
begin
	
	if ( @id is NULL ) begin
	
		if exists ( Select 1 
						From dbo.A_Scripts 
						Where ([type] = @type)
							and ([idproject] = @idproject)
							and ([name] = @name))
		begin
			
			Select @id = id
			From dbo.A_Scripts
			Where (([type] = @type)
					and ([idproject] = @idproject)
					and ([name] = @name))
					
			Update dbo.A_Scripts
			Set [modifieduserid] = @modifieduserid
			Where (([type] = @type)
					and ([idproject] = @idproject)
					and ([name] = @name))
				and ((modifieduserid is null) or (modifieduserid != @modifieduserid))
					
		end else begin					
			
			Set @id = NewID()
			
			Insert into dbo.A_Scripts ( id, name, type, idproject, modifieduserid )
			Values ( @id, @name, @type, @idproject, @modifieduserid )
			
		end
		
	end	else if exists ( Select 1 From dbo.A_Scripts Where id = @id ) begin
	
		Update dbo.A_Scripts
		Set [name] = @name,
			[type] = @type,
			[idproject] = @idproject,
			[modifieduserid] = @modifieduserid
		Where (id = @id)
			and ((name != @name)
				or (type != @type)
				or (idproject != @idproject)
				or ((modifieduserid is null) or (modifieduserid != @modifieduserid)))
				
	end else begin
			
		Insert into dbo.A_Scripts ( id, name, type, idproject )
		Values ( @id, @name, @type, @idproject )
	
	end
	
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии
	*/
	if (@modifieduserid = '00000000-0000-0000-0000-000000000000') 
		Set @modifieduserid = NULL
		
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 106,
		@name = @name,
		@idParentProject = @idproject,
		@actionUserId = @modifieduserid
	
end
GO

------------------------