﻿use oktell_settings
if (not object_id('dbo.A_SmsTaskManager_Tasks') is null) 
	if not exists (Select * From syscolumns where (id = object_id ( 'dbo.A_SmsTaskManager_Tasks' )) 
										and (name like 'DlgScript'))
		Alter Table dbo.A_SmsTaskManager_Tasks Add DlgScript nvarchar(500) not null Default ''
GO

use oktell
GO

if (not object_id('dbo.A_SmsTaskManager_Tasks') is null) 
	Drop view dbo.A_SmsTaskManager_Tasks
GO

Create view dbo.A_SmsTaskManager_Tasks
as 
	Select *
	From oktell_settings.dbo.A_SmsTaskManager_Tasks
GO


if not (object_id('dbo.A_SmsTaskManager_Task_Save') is null)
	Drop procedure dbo.A_SmsTaskManager_Task_Save
GO

Create Procedure [dbo].[A_SmsTaskManager_Task_Save]
	@id 				uniqueidentifier,
	@idproject			uniqueidentifier,
	@name				nvarchar(500),
	@code				nvarchar(30),
	@active				int,
	@done				int,
	@type				int,
	@svcscript			nvarchar(500) = '',
	@dlgscript			nvarchar(500) = '',
	@stopreason			int,
	@choiceoperator		int,
	@deleteaftersuccess	int,
	@isoutputtask		int,
	@resulted			int,
	@resulttype			int,
	@resultinfo			nvarchar(500),
	@resultdatetime		datetime,
	@creationuserid		uniqueidentifier,
	@creationtype		int,
	@creationdatetime	datetime,
	@startrundatetime	datetime,
	@stoprundatetime	datetime,
	@tagdouble			float,
	@listid				uniqueidentifier,
	@dontsetdone		bit,
	@activeonholidays	bit,
	@pausesynclistwaitingnewabonent int,
	@pausesynclistforcedsec		int,
	@ismultithread		bit,
	@threadcount		int,
	@incomingupdateifexists	bit,
	@iduser				uniqueidentifier = NULL
	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		

	if ( exists ( Select Id from A_SmsTaskManager_Tasks where Id = @id )) begin
	
		/* Обновляет задачу */
		Update A_SmsTaskManager_Tasks
		Set 
			IdProject			= @idproject,
			Name 				= @name,
			Code				= @code,
			Active 				= @active, 
			Done 				= @done, 
			Type 				= @type, 
			SvcScript 			= @svcscript, 
			StopReason 			= @stopreason, 
			ChoiceOperator		= @choiceoperator,
			DeleteAfterSuccess 	= @deleteaftersuccess, 
			IsOutputTask		= @isoutputtask,
			Resulted 			= @resulted, 
			ResultType 			= @resulttype, 
			ResultInfo 			= @resultinfo, 
			ResultDateTime 		= @resultdatetime, 
			CreationUserId 		= @creationuserid, 
			CreationType 		= @creationtype, 
			CreationDateTime 	= @creationdatetime,
			StartRunDateTime	= @startrundatetime,
			StopRunDateTime		= @stoprundatetime,
			TagDouble			= @tagdouble,	
			ListId				= @listid,
			DontSetDone			= @dontsetdone,
			ActiveOnHolidays	= @activeonholidays,
			PauseSyncListWaitingNewAbonent	= @pausesynclistwaitingnewabonent,
			PauseSyncListForcedSec		= @pausesynclistforcedsec,
			IsMultiThread		= @ismultithread,
			ThreadCount			= @threadcount,
			IncomingUpdateIfExists		= @incomingupdateifexists,
			DlgScript			= @dlgscript
 		Where
			Id = @id
	
	end	else begin
		
		Insert into A_SmsTaskManager_Tasks
		(
			Id, 
			IdProject,
			Name, 
			Code,
			Active, 
			Done, 
			Type, 
			SvcScript, 
			StopReason, 
			ChoiceOperator, 
			DeleteAfterSuccess, 
			IsOutputTask, 
			Resulted, 
			ResultType, 
			ResultInfo, 
			ResultDateTime, 
			CreationUserId, 
			CreationType, 
			CreationDateTime, 
			StartRunDateTime, 
			StopRunDateTime, 
			TagDouble, 
			ListId,
			DontSetDone,
			ActiveOnHolidays,
			PauseSyncListWaitingNewAbonent,
			PauseSyncListForcedSec,
			IsMultiThread,
			ThreadCount,
			IncomingUpdateIfExists,
			DlgScript
		) 
		Values  
		(
			@id,
			@idproject,
			@name,
			@code,
			@active,
			@done,
			@type,
			@svcscript,
			@stopreason,
			@choiceoperator,
			@deleteaftersuccess,
			@isoutputtask,
			@resulted,
			@resulttype,
			@resultinfo,
			@resultdatetime,
			@creationuserid,
			@creationtype,
			@creationdatetime,
			@startrundatetime,
			@stoprundatetime,
			@tagdouble,
			@listid,
			@dontsetdone,
			@activeonholidays,
			@pausesynclistwaitingnewabonent,
			@pausesynclistforcedsec,
			@ismultithread,
			@threadcount,
			@incomingupdateifexists,
			@dlgscript
		)		
	end
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии 
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 102,
		@name = @name,
		@idParentProject = @idproject,
		@actionUserId = @iduser
			
	--if ( @type != 0 ) begin --Auto не нужны	
	--	-- Заказ на обновление кэша групп пользователей (операторы супервизоры новые появились)
	--	exec A_Policy_SubjectGroupCache_SetUpdate		
	--end
		
end
GO


if not (object_id('dbo.A_SmsTaskManager_AllTask_Get') is null)
	Drop procedure dbo.A_SmsTaskManager_AllTask_Get
GO

Create Procedure [dbo].[A_SmsTaskManager_AllTask_Get]
as

begin
	Select 	
		Id,
		IdProject,
		Name,
		Code,		
		Active,
		Done,
		Type,
		SvcScript,
		StopReason,
		ChoiceOperator,
		DeleteAfterSuccess,
		IsOutputTask,
		Resulted,
		ResultType,
		ResultInfo,
		ResultDateTime,
		CreationUserId,
		CreationType,
		CreationDateTime,
		StartRunDateTime,
		StopRunDateTime,
		TagDouble,
		ListId,
		DontSetDone,
		ActiveOnHolidays,
		PauseSyncListWaitingNewAbonent,
		PauseSyncListForcedSec,
		IsMultiThread,
		ThreadCount,
		IncomingUpdateIfExists,
		DlgScript

	From A_SmsTaskManager_Tasks
end
GO